var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useEffect, useState } from 'react';
import ReactDatePicker, { registerLocale } from 'react-datepicker';
import enGb from 'date-fns/locale/en-GB';
import { format, getSeconds, getHours, getMinutes, set } from 'date-fns';
import styled from 'styled-components';
import { DatePickerStyle } from "../../global";
import { Button } from "../../inputs/Button";
import { Icon } from '../Icon';
import { PredefinedDates } from './PredefinedDates';
import { TimeControl } from './TimeControl';
import { getEndOfDay, getStartOfDay, isSameDate, isValidDateRange, } from './utils';
import { TIME_FORMAT, DATE_FORMAT } from './consts';
var DATEPICKER_BORDER_RADIUS = '1.25em';
var DatePickerComponent = function (_a) {
    var initialStartDate = _a.initialStartDate, initialEndDate = _a.initialEndDate, onChange = _a.onChange;
    var _b = useState(false), open = _b[0], setOpen = _b[1];
    var _c = useState(null), startDate = _c[0], setStartDate = _c[1];
    var _d = useState(null), endDate = _d[0], setEndDate = _d[1];
    var _e = useState(null), prevEndDate = _e[0], setPrevEndDate = _e[1];
    var _f = useState(false), predefinedDate = _f[0], setPredefinedDate = _f[1];
    var _g = useState(true), timeValid = _g[0], setTimeValid = _g[1];
    var currentDate = new Date();
    var localeName = 'en-gb';
    var valid = isValidDateRange(startDate, endDate) && timeValid;
    useEffect(function () {
        registerLocale(localeName, enGb);
    }, []);
    useEffect(function () {
        init();
    }, [initialStartDate, initialEndDate]);
    var formatInitialDate = function (date) {
        return typeof date === 'number' ? new Date(date) : date;
    };
    var init = function () {
        setStartDate(formatInitialDate(initialStartDate));
        setEndDate(formatInitialDate(initialEndDate));
    };
    var handleDateChange = function (date) {
        var start = date[0], end = date[1];
        setStartDate(predefinedDate ? getStartOfDay(start) : start);
        setEndDate(end ? getEndOfDay(end) : null);
        // update time
        if (!predefinedDate && end && prevEndDate) {
            var hours = getHours(prevEndDate);
            var minutes = getMinutes(prevEndDate);
            var seconds = getSeconds(prevEndDate);
            var endDate_1 = set(end, {
                hours: hours,
                minutes: minutes,
                seconds: seconds,
            });
            setEndDate(endDate_1);
        }
        setPredefinedDate(false);
    };
    var handlePredefinedDateChange = function (_a) {
        var startDate = _a.startDate, endDate = _a.endDate;
        setStartDate(startDate);
        setEndDate(endDate);
        setPredefinedDate(true);
        setPrevEndDate(null);
        setTimeValid(true);
    };
    var handleTimeChange = function (_a) {
        var startDate = _a.startDate, endDate = _a.endDate;
        setStartDate(startDate);
        setEndDate(endDate);
        setPrevEndDate(endDate);
        setPredefinedDate(false);
    };
    var handleTimeClear = function () {
        setStartDate(startDate ? getStartOfDay(startDate) : null);
        setEndDate(endDate ? getEndOfDay(endDate) : null);
        setPrevEndDate(null);
        setPredefinedDate(false);
    };
    var handleSave = function () {
        var startDateMs = startDate ? startDate.getTime() : startDate;
        var endDateMs = endDate ? endDate.getTime() : endDate;
        onChange({ startDate: startDateMs, endDate: endDateMs });
        setOpen(false);
    };
    var handleCancel = function () {
        init();
        setPrevEndDate(null);
        setTimeValid(true);
        setOpen(false);
    };
    var handleClearDate = function () {
        setStartDate(null);
        setEndDate(null);
        setPrevEndDate(null);
        setPredefinedDate(false);
        onChange({ startDate: null, endDate: null });
    };
    var openDatePicker = function () { return setOpen(true); };
    var formatWeekDay = function (d) { return d.substring(0, 3); };
    var getTriggerButtonText = function () {
        if (startDate && endDate) {
            var startDateFormatted = format(startDate, DATE_FORMAT);
            var endDateFormatted = format(endDate, DATE_FORMAT);
            var startTime = format(startDate, TIME_FORMAT);
            var endTime = format(endDate, TIME_FORMAT);
            return startDateFormatted + "/" + startTime + " - " + (isSameDate(startDate, endDate) ? '' : endDateFormatted + '/') + endTime;
        }
        return 'Set an exact time';
    };
    var renderDayContent = function (dayOfMonth) {
        return React.createElement("div", null, dayOfMonth);
    };
    return (React.createElement(DatePickerContainer, null,
        React.createElement(DatePickerControlWrapper, null,
            React.createElement(DatePickerControl, { isEmpty: !startDate && !endDate, onClick: openDatePicker }, getTriggerButtonText()),
            startDate && endDate ? (React.createElement(DatePickerControlIcon, { onClick: handleClearDate },
                React.createElement(Icon, { name: "times", fitted: true }))) : (React.createElement(DatePickerControlIcon, { onClick: openDatePicker },
                React.createElement(Icon, { name: "calendar-alt", fitted: true })))),
        open && (React.createElement(DatePickerModal, null,
            React.createElement(DatePickerCloseButton, { onClick: handleCancel },
                React.createElement(Icon, { name: "times", fitted: true })),
            React.createElement(DatePickerRow, null,
                React.createElement(DatePickerCalendar, null,
                    React.createElement(PredefinedDates, { startDate: startDate, endDate: endDate, onChange: handlePredefinedDateChange }),
                    React.createElement(ReactDatePicker, { inline: true, selected: startDate, openToDate: endDate || currentDate, onChange: handleDateChange, startDate: startDate, endDate: endDate, selectsRange: true, locale: localeName, monthsShown: 2, renderDayContents: renderDayContent, maxDate: currentDate, disabledKeyboardNavigation // removes highlighted day
                        : true, formatWeekDay: formatWeekDay, previousMonthButtonLabel: "", nextMonthButtonLabel: "" }))),
            React.createElement(DatePickerRow, null,
                React.createElement(TimeControl, { startDate: startDate, endDate: endDate, onChange: handleTimeChange, onChangeRaw: setTimeValid, onClear: handleTimeClear, predefinedTime: predefinedDate })),
            React.createElement(DatePickerFooter, null,
                React.createElement(Button, { secondary: true, onClick: handleCancel }, "Cancel"),
                React.createElement(DatePickerSaveButtonWrapper, null,
                    startDate && !endDate && (React.createElement(SaveButtonHelper, null, "Please, select the end of timeframe")),
                    React.createElement(Button, { disabled: !valid, onClick: handleSave }, "Save")))))));
};
var DatePickerModal = styled('div').withConfig({ displayName: "DatePickerModal", componentId: "sc-1gp7oak" })(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n  min-width: 870px;\n  position: absolute;\n  top: 0;\n  background: #ffffff\n    linear-gradient(180deg, #f2f2f2 0%, #ffffff 13%, #ffffff 100%) 0 0 no-repeat;\n  box-shadow: 0 30px 80px rgba(0, 0, 0, 0.05);\n  padding: 20px 30px;\n  display: flex;\n  flex-direction: column;\n  border-radius: ", ";\n  z-index: 9999;\n"], ["\n  min-width: 870px;\n  position: absolute;\n  top: 0;\n  background: #ffffff\n    linear-gradient(180deg, #f2f2f2 0%, #ffffff 13%, #ffffff 100%) 0 0 no-repeat;\n  box-shadow: 0 30px 80px rgba(0, 0, 0, 0.05);\n  padding: 20px 30px;\n  display: flex;\n  flex-direction: column;\n  border-radius: ", ";\n  z-index: 9999;\n"])), DATEPICKER_BORDER_RADIUS);
var DatePickerRow = styled('div').withConfig({ displayName: "DatePickerRow", componentId: "sc-itg20j" })(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n  display: flex;\n  margin-bottom: 20px;\n  border-bottom: 1px solid #c7c7cb;\n"], ["\n  display: flex;\n  margin-bottom: 20px;\n  border-bottom: 1px solid #c7c7cb;\n"])));
var DatePickerContainer = styled('div').withConfig({ displayName: "DatePickerContainer", componentId: "sc-qxcfg" })(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n  position: relative;\n"], ["\n  position: relative;\n"])));
var DatePickerControlWrapper = styled('div').withConfig({ displayName: "DatePickerControlWrapper", componentId: "sc-itsahb" })(templateObject_4 || (templateObject_4 = __makeTemplateObject(["\n  display: flex;\n  align-items: center;\n  width: 330px;\n  border: 1px solid ", ";\n  border-radius: ", ";\n  height: 35px;\n"], ["\n  display: flex;\n  align-items: center;\n  width: 330px;\n  border: 1px solid ", ";\n  border-radius: ", ";\n  height: 35px;\n"])), function (_a) {
    var theme = _a.theme;
    return theme.greyscale.dark;
}, DATEPICKER_BORDER_RADIUS);
var DatePickerControl = styled('button').withConfig({ displayName: "DatePickerControl", componentId: "sc-ajfwi" })(templateObject_5 || (templateObject_5 = __makeTemplateObject(["\n  cursor: pointer;\n  background: transparent;\n  border: none;\n  outline: none;\n  font-size: 14px;\n  flex-grow: 1;\n  text-align: left;\n  white-space: nowrap;\n  height: 35px;\n  border-bottom-left-radius: ", ";\n  border-top-left-radius: ", ";\n  padding: 0 10px;\n\n  ", "\n"], ["\n  cursor: pointer;\n  background: transparent;\n  border: none;\n  outline: none;\n  font-size: 14px;\n  flex-grow: 1;\n  text-align: left;\n  white-space: nowrap;\n  height: 35px;\n  border-bottom-left-radius: ", ";\n  border-top-left-radius: ", ";\n  padding: 0 10px;\n\n  ",
    "\n"])), DATEPICKER_BORDER_RADIUS, DATEPICKER_BORDER_RADIUS, function (_a) {
    var isEmpty = _a.isEmpty;
    return isEmpty && {
        fontStyle: 'italic',
    };
});
var DatePickerControlIcon = styled('button').withConfig({ displayName: "DatePickerControlIcon", componentId: "sc-1bzwy49" })(templateObject_6 || (templateObject_6 = __makeTemplateObject(["\n  cursor: pointer;\n  background: transparent;\n  border: none;\n  outline: none;\n  display: flex;\n  align-items: center;\n  justify-content: space-between;\n  flex-grow: 0;\n  height: 35px;\n  border-bottom-right-radius: ", ";\n  border-top-right-radius: ", ";\n  padding: 0 10px;\n"], ["\n  cursor: pointer;\n  background: transparent;\n  border: none;\n  outline: none;\n  display: flex;\n  align-items: center;\n  justify-content: space-between;\n  flex-grow: 0;\n  height: 35px;\n  border-bottom-right-radius: ", ";\n  border-top-right-radius: ", ";\n  padding: 0 10px;\n"])), DATEPICKER_BORDER_RADIUS, DATEPICKER_BORDER_RADIUS);
var DatePickerFooter = styled('footer').withConfig({ displayName: "DatePickerFooter", componentId: "sc-14fb7z7" })(templateObject_7 || (templateObject_7 = __makeTemplateObject(["\n  display: flex;\n  justify-content: space-between;\n  align-items: center;\n"], ["\n  display: flex;\n  justify-content: space-between;\n  align-items: center;\n"])));
var DatePickerCloseButton = styled('button').withConfig({ displayName: "DatePickerCloseButton", componentId: "sc-d0jd1j" })(templateObject_8 || (templateObject_8 = __makeTemplateObject(["\n  cursor: pointer;\n  position: absolute;\n  right: 5px;\n  top: 5px;\n  background: transparent;\n  border: none;\n  outline: none;\n  padding: 5px;\n"], ["\n  cursor: pointer;\n  position: absolute;\n  right: 5px;\n  top: 5px;\n  background: transparent;\n  border: none;\n  outline: none;\n  padding: 5px;\n"])));
var DatePickerSaveButtonWrapper = styled('div').withConfig({ displayName: "DatePickerSaveButtonWrapper", componentId: "sc-1vzjo10" })(templateObject_9 || (templateObject_9 = __makeTemplateObject(["\n  display: flex;\n  align-items: center;\n"], ["\n  display: flex;\n  align-items: center;\n"])));
var SaveButtonHelper = styled('span').withConfig({ displayName: "SaveButtonHelper", componentId: "sc-1vggdad" })(templateObject_10 || (templateObject_10 = __makeTemplateObject(["\n  margin-right: 20px;\n  font-style: italic;\n  font-size: 12px;\n  white-space: nowrap;\n"], ["\n  margin-right: 20px;\n  font-style: italic;\n  font-size: 12px;\n  white-space: nowrap;\n"])));
var DatePickerCalendar = styled('div').withConfig({ displayName: "DatePickerCalendar", componentId: "sc-fregh5" })(templateObject_11 || (templateObject_11 = __makeTemplateObject(["\n  display: flex;\n  height: 390px; // modal height changes because of month height, so set specific height\n"], ["\n  display: flex;\n  height: 390px; // modal height changes because of month height, so set specific height\n"])));
var DatePickerWrapper = styled(DatePickerComponent).withConfig({ displayName: "DatePickerWrapper", componentId: "sc-1y1mm7j" })(templateObject_12 || (templateObject_12 = __makeTemplateObject(["\n  position: relative;\n"], ["\n  position: relative;\n"])));
export var DatePicker = function (props) { return (React.createElement(React.Fragment, null,
    React.createElement(DatePickerWrapper, __assign({}, props)),
    React.createElement(DatePickerStyle, null))); };
var templateObject_1, templateObject_2, templateObject_3, templateObject_4, templateObject_5, templateObject_6, templateObject_7, templateObject_8, templateObject_9, templateObject_10, templateObject_11, templateObject_12;
